{*******************************************************}
{File:      NCOciBuff.PAS                               }
{Revision:  0.02.01 / 01.11.1999                        }
{Comment:   NC OCI8 VCL: Oracle8 error handling dialog  }
{Copyright: Portions (c) 1999, Dmitry Arefiev           }
{Author:    Dmitry Arefiev, dmitrya@inthink.com         }
{           Based on Borland Delphi DBExcept.pas        }
{*******************************************************}
{$I NCOciDef.inc}

unit NCOciErrorDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, NCOciWrapper;

type
  TNCOciErrorFrm = class(TForm)
    BasicPanel: TPanel;
    DetailsPanel: TPanel;
    NativeLabel: TLabel;
    DbMessageText: TMemo;
    NativeResult: TEdit;
    ButtonPanel: TPanel;
    IconPanel: TPanel;
    IconImage: TImage;
    TopPanel: TPanel;
    ErrorText: TLabel;
    RightPanel: TPanel;
    OKBtn: TBitBtn;
    DetailsBtn: TBitBtn;
    NextBtn: TBitBtn;
    BackBtn: TBitBtn;
    Label1: TLabel;
    ServerObject: TEdit;
    procedure FormShow(Sender: TObject);
    procedure BackClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DetailsBtnClick(Sender: TObject);
  private
    FPrevOnException: TExceptionEvent;
    FOCINativeError: EOCINativeError;
    FDetailsHeight, CurItem: Integer;
    FDetails: string;
    procedure HandleException(Sender: TObject; E: Exception);
    procedure SwitchDetails;
    procedure ShowError;
    function ShowException(Error: EOCINativeError): TModalResult;
    property OCINativeError: EOCINativeError read FOCINativeError write FOCINativeError;
  public
    class procedure HookExceptions;
    class procedure UnHookExceptions;
  end;

var
  NCOciErrorFrm: TNCOciErrorFrm;

implementation

{$R *.DFM}

procedure TNCOciErrorFrm.HandleException(Sender: TObject; E: Exception);
begin
  if (E is EOCINativeError) and (OCINativeError = nil) and
    not Application.Terminated then ShowException(EOCINativeError(E))
  else if Assigned(FPrevOnException) then FPrevOnException(Sender, E)
  else Application.ShowException(E);
end;

procedure TNCOciErrorFrm.SwitchDetails;
const
  DetailsOn: array [Boolean] of string = ('%s >>', '<< %s');
var
  DEnabling: Boolean;
begin
  DEnabling := not DetailsPanel.Visible;
  if DEnabling then ClientHeight := FDetailsHeight
  else ClientHeight := DetailsPanel.Top;
  DetailsPanel.Visible := DEnabling;
  ButtonPanel.Top := 0;
  DetailsBtn.Caption := Format(DetailsOn[DEnabling], [FDetails]);
end;

procedure TNCOciErrorFrm.ShowError;
var
  errMsg: TOCIErrorMessage;
  s: String;
  i, i1, i2: Integer;
begin
  BackBtn.Enabled := CurItem > 0;
  NextBtn.Enabled := CurItem < OCINativeError.ErrorCount - 1;
  if OCINativeError.ErrorCount > 0 then begin
    errMsg := OCINativeError.Errors[CurItem];
    DbMessageText.Text := errMsg.Message;
    s := IntToStr(errMsg.ErrorCode);
    for i := 1 to 5 - Length(s) do
        s := '0' + s;
    NativeResult.Text := s;
    s := errMsg.Message;
    i1 := Pos('#13', s);
    i2 := Pos('#10', s);
    if (i2 <> 0) and (i2 < i1) or (i1 = 0) then
        i1 := i2;
    if i1 = 0 then
        i1 := Length(s) + 1;
    s := Copy(s, 1, i1 - 1);
    i1 := Pos('(', s);
    i2 := Pos(')', s);
    ServerObject.Text := Copy(s, i1 + 1, i2 - i1 - 1);
  end
  else begin
    DbMessageText.Text := '';
    NativeResult.Text := '';
    ServerObject.Text := '';
  end;
end;

procedure TNCOciErrorFrm.FormCreate(Sender: TObject);
begin
  FDetailsHeight := ClientHeight;
  FDetails := DetailsBtn.Caption;
  SwitchDetails;
end;

procedure TNCOciErrorFrm.FormShow(Sender: TObject);
begin
  ErrorText.Caption := OCINativeError.Message;
  if DetailsPanel.Visible then
  begin
    CurItem := 0;
    ShowError;
  end;
end;

procedure TNCOciErrorFrm.BackClick(Sender: TObject);
begin
  Dec(CurItem);
  ShowError;
end;

procedure TNCOciErrorFrm.NextClick(Sender: TObject);
begin
  Inc(CurItem);
  ShowError;
end;

procedure TNCOciErrorFrm.DetailsBtnClick(Sender: TObject);
begin
  SwitchDetails;
  if DetailsPanel.Visible then
  begin
    CurItem := 0;
    ShowError;
  end;
end;

class procedure TNCOciErrorFrm.HookExceptions;
begin
  if NCOciErrorFrm = nil then begin
    NCOciErrorFrm := TNCOciErrorFrm.Create(nil);
    NCOciErrorFrm.FPrevOnException := Application.OnException;
    Application.OnException := NCOciErrorFrm.HandleException;
  end;
end;

class procedure TNCOciErrorFrm.UnHookExceptions;
begin
  if NCOciErrorFrm <> nil then begin
    Application.OnException := NCOciErrorFrm.FPrevOnException;
    NCOciErrorFrm.Free;
    NCOciErrorFrm := nil;
  end;
end;

function TNCOciErrorFrm.ShowException(Error: EOCINativeError): TModalResult;
begin
  OCINativeError := Error;
  Result := ShowModal;
  OCINativeError := nil;
end;

initialization
    NCOciErrorFrm := nil;

finalization
    TNCOciErrorFrm.UnHookExceptions;

end.
